import pygame

from BrickBreaker.Scenes.scene import Scene
from BrickBreaker.Shared import *


class PlayingGameScene(Scene):

    def __init__(self, game):
        super().__init__(game)

    def render(self):
        super().render()

        game = self.get_game()
        level = game.get_level()
        pad = game.get_pad()
        balls = game.get_balls()

        if level.get_amount_of_bricks_left() <= 0:
            for ball in balls:
                ball.set_motion(0)
                ball.update_position()

            level.load_next_level()

        if game.get_lives() <= 0:
            game.play_sound(GameConstants.SOUND_GAME_OVER)
            game.change_scene(GameConstants.GAME_OVER_SCENE)

        for ball in balls:
            for ball2 in balls:
                if ball != ball2 and ball.intersects(ball2):
                    ball.change_direction(ball2)

            for brick in game.get_level().get_bricks():
                if not brick.is_destroyed() and ball.intersects(brick):
                    game.play_sound(brick.get_hit_sound())
                    brick.hit()
                    level.brick_hit()
                    if brick.get_hit_sound() == 1:
                        game.increment_bonus()
                    game.increase_score(brick.get_hit_points())
                    ball.change_direction(brick)
                    break

            if ball.intersects(pad):
                game.play_sound(GameConstants.SOUND_HIT_WALL_OR_PAD)
                ball.change_direction(pad)

            ball.update_position()

            if ball.is_ball_dead():
                if len(balls) <= 1:
                    ball.set_motion(0)
                    game.reduce_life_by_one()
                    game.reset_bonus()
                    game.reset_pad()
                else:
                    balls.remove(ball)

            game.screen.blit(ball.get_image(), ball.get_position())

        for brick in game.get_level().get_bricks():
            if not brick.is_destroyed():
                game.screen.blit(brick.get_image(), brick.get_position())

        if pad.get_keyboard_status():

            pad.set_position((pad.get_position()[0], pad.get_position()[1]))
            if pad.right_is_pressed:
                pad.set_position((pad.get_position()[0] + 15, pad.get_position()[1]))
            if pad.left_is_pressed:
                pad.set_position((pad.get_position()[0] - 15, pad.get_position()[1]))

        if pad.get_mouse_status():
            pad.set_position((pygame.mouse.get_pos()[0], pad.get_position()[1]))

        game.screen.blit(pad.get_image(), pad.get_position())

        self.clear_text()

        self.add_text("Your score: " + str(game.get_score()), x=0, y=GameConstants.SCREEN_SIZE[1] - 60, size=30)

        self.add_text("Lives: " + str(game.get_lives()), x=0, y=GameConstants.SCREEN_SIZE[1] - 30, size=30)

        self.add_text(f"Bonus: x{str(game.get_bonus())}", x=GameConstants.SCREEN_SIZE[0] - 100,
                      y=GameConstants.SCREEN_SIZE[1] - 30, size=30)

    def handle_events(self, events):
        super().handle_events(events)
        game = self.get_game()
        pad = game.get_pad()

        for event in events:
            if event.type == pygame.QUIT:
                exit()

            if event.type == pygame.MOUSEBUTTONDOWN:
                for ball in self.get_game().get_balls():
                    ball.set_motion(1)

            if event.type == pygame.KEYDOWN:
                if event.key == pygame.K_SPACE:
                    for ball in self.get_game().get_balls():
                        ball.set_motion(1)

            if event.type == pygame.KEYDOWN:
                if event.key == pygame.K_LEFT:
                    pad.left_pressed(True)
                if event.key == pygame.K_RIGHT:
                    pad.right_pressed(True)

            if event.type == pygame.KEYUP:
                if event.key == pygame.K_LEFT:
                    pad.left_pressed(False)
                if event.key == pygame.K_RIGHT:
                    pad.right_pressed(False)



from BrickBreaker.ball import Ball
from BrickBreaker.level import Level
from BrickBreaker.pad import Pad
from BrickBreaker.double_pad import DoublePad
from BrickBreaker.brick_breaker import BrickBreaker



import pygame


class Scene:

    def __init__(self, game):
        self._game = game
        self._texts = []

    def render(self):
        for text in self._texts:
            self._game.screen.blit(text[0], text[1])

    def get_game(self):
        return self._game

    def handle_events(self, events):
        pass

    def clear_text(self):
        self._texts = []

    def add_text(self, string, x=0, y=0, color=[255, 255, 255], background=[0, 0, 0], size=17, font_type=None,
                 italic=False, bold=False):
        font = pygame.font.SysFont(font_type, size, bold=bold, italic=italic)
        self._texts.append([font.render(string, True, color, background), (x, y)])



from BrickBreaker.Shared import GameObject
from BrickBreaker.Shared import GameConstants


class Brick(GameObject):

    def __init__(self, position, image, game):
        self._game = game
        self._hit_points = 100
        self._lives = 1

        super().__init__(position, GameConstants.BRICK_SIZE, image)

    def get_game(self):
        return self._game

    def is_destroyed(self):
        return self._lives < 1

    def get_hit_points(self):
        return self._hit_points

    def hit(self):
        self._lives -= 1

    @staticmethod
    def get_hit_sound():
        return GameConstants.SOUND_HIT_BRICK



from BrickBreaker.Bricks.brick import *
from BrickBreaker.Bricks.life_brick import *
from BrickBreaker.Bricks.speed_brick import *
from BrickBreaker.Bricks.size_brick import *
from BrickBreaker.Bricks.ball_brick import *



from BrickBreaker.Bricks import Brick
from BrickBreaker.Shared import *


class BallBrick(Brick):

    def __init__(self, position, image, game):
        super().__init__(position, image, game)

    def hit(self):
        game = self.get_game()
        game.add_one_ball()

        super().hit()

    @staticmethod
    def get_hit_sound():
        return GameConstants.SOUND_HIT_EXTRA_BALL



import pygame

from BrickBreaker import *
from BrickBreaker.Scenes import *
from BrickBreaker.Shared import *


class BrickBreaker:

    def __init__(self):
        self._lives = 5
        self._score = 0
        self._bonus = 1

        self._level = Level(self)
        self._level.load_random()

        self._pad = Pad((GameConstants.SCREEN_SIZE[0] / 2,
                        GameConstants.SCREEN_SIZE[1] - GameConstants.PAD_SIZE[1]),
                        pygame.image.load(GameConstants.PAD_IMAGE))

        self._balls = [
            Ball((400, 400), pygame.image.load(GameConstants.BALL_IMAGE), self)
        ]

        pygame.mixer.pre_init(44100, -16, 2, 2048)
        pygame.mixer.init()
        pygame.init()
        pygame.display.set_caption("Brick Breaker")

        self._clock = pygame.time.Clock()

        self.screen = pygame.display.set_mode(GameConstants.SCREEN_SIZE)

        pygame.mouse.set_visible(False)

        self._scenes = (
            PlayingGameScene(self),
            MainMenuScene(self),
            GameOverScene(self),
            WinScene(self),
            ControlsScene(self),
            GameRulesScene(self),
        )

        self._current_scene = 1

        self._sounds = (
            pygame.mixer.Sound(GameConstants.SOUND_FILE_HITTING_A_STANDARD_BRICK),
            pygame.mixer.Sound(GameConstants.SOUND_FILE_HITTING_SPEED_UP_BRICK),
            pygame.mixer.Sound(GameConstants.SOUND_FILE_HITTING_EXTRA_LIFE_BRICK),
            pygame.mixer.Sound(GameConstants.SOUND_FILE_BALL_HITTING_A_WALL_OR_A_PAD),
            pygame.mixer.Sound(GameConstants.SOUND_FILE_GAME_OVER),
            pygame.mixer.Sound(GameConstants.SOUND_FILE_HITTING_EXTRA_BALL_BRICK),
            pygame.mixer.Sound(GameConstants.SOUND_FILE_HITTING_BONUS_SIZE_BRICK),
        )

    def start(self):
        while True:
            self._clock.tick(60)

            self.screen.fill((0, 0, 0))

            _current_scene = self._scenes[self._current_scene]
            _current_scene.handle_events(pygame.event.get())
            _current_scene.render()

            pygame.display.update()

    def change_scene(self, scene):
        self._current_scene = scene

    def get_level(self):
        return self._level

    def get_bonus(self):
        return self._bonus

    def increment_bonus(self):
        self._bonus += 1

    def reset_bonus(self):
        self._bonus = 1

    def double_pad(self):
        self._pad = DoublePad((GameConstants.SCREEN_SIZE[0] / 2,
                              GameConstants.SCREEN_SIZE[1] - GameConstants.DOUBLE_PAD_SIZE[1]),
                              pygame.image.load(GameConstants.DOUBLE_PAD_IMAGE))

    def reset_pad(self):
        keyboard = self._pad.get_keyboard_status()
        mouse = self._pad.get_mouse_status()
        self._pad = Pad((GameConstants.SCREEN_SIZE[0] / 2,
                         GameConstants.SCREEN_SIZE[1] - GameConstants.PAD_SIZE[1]),
                        pygame.image.load(GameConstants.PAD_IMAGE))
        if keyboard:
            self._pad.activate_keyboard()
        if mouse:
            self._pad.activate_mouse()

    def get_pad(self):
        return self._pad

    def get_score(self):
        return self._score

    def increase_score(self, score):
        self._score += score * self._bonus

    def increase_score_by_1k(self, score=1000):
        self._score += score * self._bonus

    def get_lives(self):
        return self._lives

    def get_balls(self):
        return self._balls

    def add_one_ball(self):
        self._balls.append(Ball((400, 400), pygame.image.load(GameConstants.BALL_IMAGE), self))

    def play_sound(self, sound_clip):
        sound = self._sounds[sound_clip]

        sound.stop()
        sound.play()

    def reduce_life_by_one(self):
        self._lives -= 1

    def add_one_life(self):
        self._lives += 1

    def reset(self):
        self._lives = 5
        self._score = 0
        self._bonus = 1
        self._level = Level(self)
        self._level.load_random()

        self._pad = Pad((GameConstants.SCREEN_SIZE[0] / 2,
                         GameConstants.SCREEN_SIZE[1] - GameConstants.PAD_SIZE[1]),
                        pygame.image.load(GameConstants.PAD_IMAGE))


def main():
    BrickBreaker().start()


if __name__ == '__main__':
    BrickBreaker().start()



from BrickBreaker.Shared import *


class Pad(GameObject):

    def __init__(self, position, image):
        self.left_is_pressed = False
        self.right_is_pressed = False
        self._mouse = False
        self._keyboard = True

        super().__init__(position, GameConstants.PAD_SIZE, image)

    def set_position(self, position):

        new_position = [position[0], position[1]]
        size = self.get_size()

        if new_position[0] + size[0] > GameConstants.SCREEN_SIZE[0]:
            new_position[0] = GameConstants.SCREEN_SIZE[0] - size[0]

        if new_position[0] < 0:
            new_position[0] = 0

        super().set_position(new_position)

    def left_pressed(self, pressed):
        self.left_is_pressed = pressed

    def right_pressed(self, pressed):
        self.right_is_pressed = pressed

    def activate_mouse(self):
        self._mouse = True
        self._keyboard = False

    def activate_keyboard(self):
        self._mouse = False
        self._keyboard = True

    def get_mouse_status(self):
        return self._mouse

    def get_keyboard_status(self):
        return self._keyboard



from BrickBreaker.Bricks import Brick
from BrickBreaker.Shared import *
from BrickBreaker.pad import Pad


class SizeBrick(Brick):

    def __init__(self, position, image, game):
        super().__init__(position, image, game)

    def hit(self):
        game = self.get_game()
        if isinstance(game.get_pad(), Pad):
            game.double_pad()
        else:
            game.increase_score_by_1k()

        super().hit()

    @staticmethod
    def get_hit_sound():
        return GameConstants.SOUND_HIT_BONUS_SIZE



"""
Brick Breaker
-------------

Brick Breaker is a simple game based on breaking bricks
and progressing through levels.

You can get it by downloading it directly or by typing:

.. code:: bash

    $ pip install BrickBreakerGame

After it is installed you can start it by simply typing in your terminal:

.. code:: bash

    $ brickbreaker

"""

from setuptools import setup


setup(name="BrickBreakerGame",
      version="0.1.5",
      description="Brick Breaker game",
      long_description=__doc__,
      url="https://github.com/Urosh91/BrickBreaker",
      author="Uros Jevremović",
      author_email="jevremovic.uros91@gmail.com",
      packages=["BrickBreaker", "BrickBreaker.Bricks", "BrickBreaker.Scenes", "BrickBreaker.Shared"],
      install_requires=["pygame"],
      include_package_data=True,
      entry_points={
          "console_scripts": ["brickbreaker=BrickBreaker.brick_breaker:main"],
      },
      )

__author__ = "Uros Jevremović"



from BrickBreaker.Bricks import Brick
from BrickBreaker.Shared import *


class SpeedBrick(Brick):

    def __init__(self, position, image, game):
        super().__init__(position, image, game)

    def hit(self):
        game = self.get_game()

        for ball in game.get_balls():
            ball.set_speed(ball.get_speed() + 1)

        super().hit()

    @staticmethod
    def get_hit_sound():
        return GameConstants.SOUND_HIT_SPEED_UP

